/**
 * Created by Andrey on 29.09.2015.
 */
var request = require('request');
var xml2js = require('xml2js');
var _ = require('underscore');
var util = require('util');

module.exports = function(config) {


    var _baseUrl = config.baseUrl;
    var _basePath = '/admin/project';
    var _cookies = config.cookies;

    this.getProject = function(params, done) {
        if (params === undefined) {
            return done('Params is not defined', false);
        }
        if (params.projectId === undefined || !params.projectId) {
            return done('projectId is required', false);
        }

        var _path = _basePath + '/' + params.projectId;

        request.get(_baseUrl + _path, {
            headers: {
                "Cookie": _cookies
            }
        }, function (err, httpRes, body) {
            if (!err && httpRes.statusCode == 200){
                xml2js.parseString(body, function (err, result) {
                    if (!err) {
                        var project;
                        project = result.project['$'];
                        done('null', project);
                    }
                });
            }
        })
    };


    this.getProjects = function(done) {

        var _path = _basePath + '/';

        request.get(_baseUrl + _path, {
            headers: {
                "Cookie": _cookies
            }
        }, function (err, httpRes, body) {
            if (!err && httpRes.statusCode == 200){
                xml2js.parseString(body, function (err, result) {
                    if (!err) {
                        var projects = [];

                        _.each(result.projectRefs.project, function (value) {
                            projects.push(value['$']);
                        });

                        if(!projects.length) {
                            console.warn('Warning: Youtrack projects is empty');
                        }

                        done('null', projects);
                    }
                });
            }
        })
    };

    // Create new project
    this.putProject = function(params, done) {

        if (params === undefined) {
            return done('Params is not defined', false);
        }
        if (params.projectId === undefined || !params.projectId) {
            return done('projectId is required', false);
        }
        if (params.projectName === undefined || !params.projectName) {
            return done('projectName is required', false);
        }
        if (params.startingNumber === undefined) {
            return done('startingNumber is required', false);
        }
        if (params.projectLeadLogin === undefined || !params.projectLeadLogin) {
            return done('projectLeadLogin is required', false);
        }


        var _path = _basePath + '/' + params.projectId;

        request.put(_baseUrl + _path, {
            headers: {
                "Cookie": _cookies
            },
            qs: {
                projectId: params.projectId,
                projectName: params.projectName,
                startingNumber: params.startingNumber,
                projectLeadLogin: params.projectLeadLogin,
                description: params.description === undefined ? '' : params.description
            }
        }, function (err, httpRes, body) {

            if (!err && httpRes.statusCode == 201){
                done('null', true);
            }
            else if (httpRes.statusCode >= 400 && httpRes.statusCode < 500){
                xml2js.parseString(body, function (err, result) {
                    done(result.error, false);
                });
            }
            else if (httpRes.statusCode == 500){
                done('Server error', false);
            }
        })
    };

    // Delete project
    this.deleteProject = function(params, done) {
        if (params === undefined) {
            return done('Params is not defined', false);
        }
        if (params.projectId === undefined || !params.projectId) {
            return done('projectId is required', false);
        }

        var _path = _basePath + '/' + params.projectId;

        request.del(_baseUrl + _path, {
            headers: {
                "Cookie": _cookies
            }
        }, function (err, httpRes, body) {

            if (!err && httpRes.statusCode == 200){
                done('null', true);
            }
            else {
                xml2js.parseString(body, function (err, result) {
                    done(result.error, false);
                });
            }
        })
    };

    this.getAssignees = function(params, done) {
        if (params === undefined) {
            return done('Params is not defined', false);
        }
        if (params.projectId === undefined || !params.projectId) {
            return done('projectId is required', false);
        }

        var _path = _basePath + '/' + params.projectId + '/assignee';

        request.get(_baseUrl + _path, {
            headers: {
                "Cookie": _cookies
            }
        }, function (err, httpRes, body) {

            var assignees = {};
            if (!err && httpRes.statusCode == 200){
                xml2js.parseString(body, function (err, result) {
                    _.extend(assignees, result.assigneeList['$']);
                    assignees.assignee = [];
                    _.each(result.assigneeList.assignees[0].assignee, function (value) {
                        value['$'].assignedBy = {};
                        _.extend(value['$'].assignedBy, value.assignedBy);
                        assignees.assignee.push(value['$']);
                    });
                    done('null', assignees);
                });
            }
            else {
                xml2js.parseString(body, function (err, result) {
                    done(result.error, 'null');
                });
            }
        })
    };
    return this;
};
